<?php

namespace MoSharePointObjectSync\API;

use MoSharePointObjectSync\Observer\adminObserver;
use MoSharePointObjectSync\Wrappers\wpWrapper;

class Authorization{
    private static $instance;

    public static function getController(){
        if(!isset(self::$instance)){
            $class = __CLASS__;
            self::$instance = new $class;
        }
        return self::$instance;
    }

    public function mo_sps_get_access_token_using_client_credentials($endpoints,$config,$scope){
        $client_secret = wpWrapper::mo_sps_decrypt_data($config['client_secret'],hash("sha256",$config['client_id']));
        $url = $config['admin_uri'];
        $domain = wpWrapper::mo_sps_get_domain_from_url($url);

        $args =  [
            'body' => [
                'grant_type' => 'client_credentials',
                'client_secret' => $client_secret,
                'client_id' => $config['client_id'].'@'.$config['tenant_id'],
                'resource'=>'00000003-0000-0ff1-ce00-000000000000/'.$domain.'@'.$config['tenant_id']
            ],
            'headers' => [
                'Content-type' => 'application/x-www-form-urlencoded'
            ]
            
        ];


        $response = wp_remote_post(esc_url_raw($endpoints['token']),$args);
       
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            wp_die("Error Occurred : ".esc_html($error_message));
        } else {
            $body= json_decode($response["body"],true);
            if(isset($body['error']) && isset($_REQUEST['option']) && ($_REQUEST['option'] == 'testSPSApp' || $_REQUEST['option'] == 'testSPSUser')){
                $observer = adminObserver::getObserver();
                $error_code = [
                    "Error" => $body['error'],
                   "Description" => $body['error_description']
               ];
               $observer->mo_sps_display_error_message($error_code);
            }
            if(isset($body["access_token"])){
                return $body["access_token"];
            }
              }
        return false;
    }

    

    public function mo_sps_get_grpah_access_token_using_client_credentials($endpoints,$config,$scope){
        $client_secret = wpWrapper::mo_sps_decrypt_data($config['client_secret'],hash("sha256",$config['client_id']));
        
        $args =  [
            'body' => [
                'grant_type' => 'client_credentials',
                'client_secret' => $client_secret,
                'client_id' => $config['client_id'],
                'scope'=>$scope
            ],
            'headers' => [
                'Content-type' => 'application/x-www-form-urlencoded'
            ]
            
        ];


        $response = wp_remote_post(esc_url_raw($endpoints['graph_token']),$args);
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            wp_die("Error Occurred : ".esc_html($error_message));
        } else {
            $body= json_decode($response["body"],true);
            if(isset($body["access_token"])){
                return $body["access_token"];
            }
        }
        return false;
    }

    public function mo_sps_get_request($url,$headers){
        $args = [
            'headers' => $headers
        ];
        $response = wp_remote_get(esc_url_raw($url),$args);
        if ( is_array( $response ) && ! is_wp_error( $response ) ) {
            return json_decode($response["body"],true);
        } else {
            wp_die("Error occurred: ".esc_html($response->get_error_message()));
        }
    }
   
}