<?php

namespace MoSharePointObjectSync\API;

use MoSharePointObjectSync\Wrappers\sharepointWrapper;
use MoSharePointObjectSync\Wrappers\wpWrapper;
use MoSharePointObjectSync\Wrappers\pluginConstants;


class Azure{

    private static $obj;
    private $endpoints;
    private $config;
    private $scope = "https://graph.microsoft.com/.default";
    private $access_token;
    private $handler;

    private function __construct($config){
        $this->config = $config;
        $this->handler = Authorization::getController();
    }

    public static function getClient($config){
        if(!isset(self::$obj)){

            self::$obj = new Azure($config);
            self::$obj->setEndpoints();

        }
        return self::$obj;
    }

    public static function mo_sps_get_endpoints(){
        return $this->setEndpoints();
    }
    
    public function mo_sps_sync_azure_users(){
        $this->access_token = sanitize_text_field($this->handler->mo_sps_get_grpah_access_token_using_client_credentials($this->endpoints,$this->config,$this->scope));
        if(!$this->access_token){
            wp_die(esc_html("Access token is missing from the response. Please try again later."));
        }
        $this->fetch_users_using_access_token();
    }

    public function mo_sps_sync_individual_azure_user($upn_id){
        $this->access_token = sanitize_text_field($this->handler->mo_sps_get_access_token_using_client_credentials($this->endpoints,$this->config,$this->scope));
        if(!$this->access_token){
            wp_die(esc_html("Access token is missing from the response. Please try again later."));
        }

        $status = $this->fetch_individual_user_using_access_token($upn_id);
        return $status;

    }

    private function setEndpoints(){
        $this->endpoints['token'] = 'https://accounts.accesscontrol.windows.net/'.$this->config['tenant_id'].'/tokens/OAuth/2';
        $this->endpoints['graph_token'] = 'https://login.microsoftonline.com/'.$this->config['tenant_id'].'/oauth2/v2.0/token';
        $this->endpoints['user'] = "https://".wpWrapper::mo_sps_get_domain_from_url($this->config['admin_uri'])."/_api/SP.UserProfiles.PeopleManager/GetPropertiesFor(accountName=@v)?@v=";
        $this->endpoints['graph_users'] = 'https://graph.microsoft.com/beta/users/?$select=userPrincipalName,id';

        $this->endpoints['documents'] = "https://".wpWrapper::mo_sps_get_domain_from_url($this->config['admin_uri'])."/_api/web/GetFolderByServerRelativeUrl('/Shared Documents')/?".'$expand=files,folders';
    }

    private function fetch_users_using_access_token(){
        $args = [
            'Authorization' => 'Bearer '.$this->access_token
        ];
        $users = $this->handler->mo_sps_get_request($this->endpoints['graph_users'],$args);

        if(!is_array($users) || count($users)<=0){
            wp_die(esc_html("Unknown error occurred. Please try again later."));
        }
        foreach ($users['value'] as $user){
            if(!empty($user)){
                $user_login = sanitize_user($user['id']);
                $user_email = sanitize_email($user['userPrincipalName']);

                $user_id = username_exists($user_login);
                if(!$user_id){
                    $user_id = email_exists($user_email);
                }
                if(!$user_id){
                    $random_pass = wp_generate_password('12', false);
                    $user_id = wp_create_user($user_login, $random_pass, $user_email);
                }else{
                    wp_update_user([
                        'user_email' => $user_email
                    ]);
                }
            }
        }
    }

    private function fetch_individual_user_using_access_token($upn_id){
        $this->access_token = $this->handler->mo_sps_get_access_token_using_client_credentials($this->endpoints,$this->config,$this->scope);
        $profile = wpWrapper::mo_sps_get_option(pluginConstants::PROFILE_MAPPING);
        $args = [
            'Authorization' => 'Bearer '.$this->access_token,
            'Accept'=>'application/json; odata=verbose'
        ];
      
        
        $user_info_url = $this->endpoints['user']."'i:0%23.f|membership|".$this->config['upn_id']."'";
       
        $user = $this->handler->mo_sps_get_request($user_info_url,$args);
       
        $user = sharepointWrapper::mo_sps_array_get_sharepoint_user_profile($user);
       
        
        if(!is_array($user) || count($user)<=0){
            wp_die(esc_html("Unknown error occurred. Please try again later."));
        }

        if(array_key_exists('error',$user)){
            wpWrapper::mo_sps__show_error_notice(esc_html__($user['error']['message']));
            return false;
        }

        
        

       
                if(!empty($user)){

                    $user_login = isset($profile['user_login'])?sanitize_user($user[$profile['user_login']]):sanitize_user($user['msOnline-ObjectId']);;
                    $user_email = isset($profile['email'])?sanitize_email($user[$profile['email']]):sanitize_email($user['UserName']);
                    $user_first_name = isset($profile['first_name'])?sanitize_text_field($user[$profile['first_name']]):sanitize_email($user['FirstName']);
                    $user_last_name = isset($profile['last_name'])?sanitize_text_field($user[$profile['last_name']]):sanitize_email($user['LastName']);
                    $user_full_name = isset($profile['display_name'])?sanitize_text_field($user[$profile['display_name']]):sanitize_email($user['PreferredName']);

                   

                    $user_id = username_exists($user_login);  
                    if(!$user_id){
                        $user_id = email_exists($user_email);
                    }

                    if(!$user_first_name){
                        $user_first = '';
                    }

                    if(!$user_last_name){
                        $user_last = '';
                    }
                    if(!$user_full_name){
                        $user_full_name = $user_first_name." ".$user_last_name;
                        if(!$user_full_name){
                            $user_full_name = '';
                        }
                    }

                    if(!$user_id){
                        $random_pass = wp_generate_password('12', false);
                        $user_id =
                         wp_insert_user(
                            [
                                'user_login'    => $user_login,
                                'user_email'    => $user_email,
                                'first_name'    => $user_first_name,
                                'last_name'     => $user_last_name,
                                'display_name'  => $user_full_name
                            ]
                        );
                        if(!is_numeric($user_id)){
                            return $user_id->errors;
                        }
                        
                        
                    }
                    wp_update_user(
                            [
                            'user_login'    => $user_login,
                            'user_email'    => $user_email,
                            'display_name'  => $user_full_name
                            ]
                        );
                  
                    
                    update_user_meta( $user_id, 'first_name', $user_first_name, '' );
                    update_user_meta( $user_id, 'last_name', $user_last_name,  '' );
                    

                }
                return true;


    }


    public function mo_sps_get_specific_user_detail(){
        $user_info_url = $this->endpoints['user']."'i:0%23.f|membership|".$this->config['upn_id']."'";
        $this->access_token = sanitize_text_field($this->handler->mo_sps_get_access_token_using_client_credentials($this->endpoints,$this->config,$this->scope));
        if(!$this->access_token){
            wp_die(esc_html("Access token is missing from the response. Please try again later."));
        }

        $args = [
            'Authorization' => 'Bearer '.$this->access_token,
            'Accept'=>'application/json; odata=verbose'
        ];
        $user = $this->handler->mo_sps_get_request($user_info_url,$args);
        
        return $user;
    }



    public function mo_sps_access_token_details(){
        $this->access_token = sanitize_text_field($this->handler->mo_sps_get_access_token_using_client_credentials($this->endpoints,$this->config,$this->scope));
        if($this->access_token){
            return true;
        }
    }

    public function mo_sps_document_array($ServerRelativeUrl){
        $app = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);
        $this->access_token = sanitize_text_field($this->handler->mo_sps_get_access_token_using_client_credentials($this->endpoints,$this->config,$this->scope));
        
 
        $args = array(
            'Authorization' => 'Bearer '.$this->access_token,
            'Accept'=>'application/json; odata=verbose'
        );


        $url = "https://".wpWrapper::mo_sps_get_domain_from_url($app['admin_uri'])."/_api/web/GetFolderByServerRelativeUrl('$ServerRelativeUrl')/?".'$expand=files,folders'.'&$expand=Editor/Id';
        $response = $this->handler->mo_sps_get_request($url,$args);


        

        return $response;
    }
}