<?php

namespace MoSharePointObjectSync\Controller;

use MoSharePointObjectSync\API\CustomerMOSPS;
use MoSharePointObjectSync\Wrappers\wpWrapper;

class accountSetup {

    private static $user;

    public static function getController(){
        if(!isset(self::$user)){
            self::$user = new accountSetup();
        }
        return self::$user;
    }

    public function mo_sps_save_settings(){
        $option = sanitize_text_field($_POST['option']);
        switch ($option){
            case 'mo_sps_account_setup_option':{
                $this->mo_sps_account_setup();
                break;
            }
            case 'mo_sps_verify_license_option':{
                $this->mo_sps_verify_license();
                break;
            }
            case 'mo_sps_go_back_from_license_page_option':{
                $this->mo_sps_go_back_from_license_page();
                break;
            }
            case 'mo_sps_remove_account_option':{
                $this->mo_sps_remove_account();
                break;
            }
        }
    }

    private function mo_saml_is_extension_installed($extension_name) {
        if  (in_array  ($extension_name, get_loaded_extensions())) {
            return 1;
        } else
            return 0;
    }

    private function mo_sps_account_setup(){
        check_admin_referer('mo_sps_account_setup_option');

        if (!$this->mo_saml_is_extension_installed('curl')) {
            wpWrapper::mo_sps__show_error_notice("ERROR: PHP cURL extension is not installed or disabled. Login failed.");
            return;
        }

        //validation and sanitization
        $email = '';
        $password = '';
        if (empty($_POST['account_email']) || empty($_POST['account_pwd'])) {
            wpWrapper::mo_sps__show_error_notice("All the fields are required. Please enter valid entries.");
            return;
        } else if (wpWrapper::mo_sps__checkPasswordPattern(strip_tags($_POST['account_pwd']))) {
            wpWrapper::mo_sps__show_error_notice("Minimum 6 characters should be present. Maximum 15 characters should be present. Only following symbols (!@#.$%^&*-_) should be present.");
            return;
        } else {
            $email = sanitize_email($_POST['account_email']);
            $password = stripslashes(strip_tags($_POST['account_pwd']));
        }

        update_option('mo_saml_admin_email', $email);
        update_option('mo_saml_admin_password', $password);
        $customer = new CustomerMOSPS();
        $content = $customer->mo_sps_get_customer_key();
        if (!$content)
            return;

        $customerKey = json_decode($content, true);

        if (json_last_error() == JSON_ERROR_NONE) {
            update_option('mo_saml_admin_customer_key', $customerKey['id']);
            update_option('mo_saml_admin_api_key', $customerKey['apiKey']);
            update_option('mo_sps_customer_token', $customerKey['token']);
            if (!empty($customerKey['phone'])) {
                update_option('mo_saml_admin_phone', $customerKey['phone']);
            }
            update_option('mo_saml_admin_password', '');
            update_option('mo_sps_registration_status', 'Existing User');
            delete_option('mo_sps_verify_customer');
            if (get_option('sps_lk')) {
                $key = get_option('mo_sps_customer_token');
                $code = wpWrapper::mo_sps_decrypt_data(get_option('sps_lk'), $key);
                $content = $customer->mo_sps_vl($code, false);
                if (!$content)
                    return;
                $content = json_decode($content, true);
                update_option('mo_sps_vl_check_t', time());
                if (strcasecmp($content['status'], 'SUCCESS') == 0) {

                    $dir = plugin_dir_path(__FILE__);

                    //home url of the site
                    $home_url = home_url();
                    $home_url = trim($home_url, '/');
                    if (!preg_match('#^http(s)?://#', $home_url)) {
                        $home_url = 'http://' . $home_url;
                    }
                    $urlParts = parse_url($home_url);
                    $domain = preg_replace('/^www\./', '', $urlParts['host']);

                    //upload path of the directory
                    $upload_dir = wp_upload_dir();
                    $asdjka = $domain . '-' . $upload_dir['basedir'];

                    //Getting hash of the domain and upload path.
                    $dmndsasd = hash_hmac('sha256', $asdjka, '4DHfjgfjasndfsajfHGJ');

                    //finding the position where we need to insert this hash
                    $fdjk = wpWrapper::djkasjdksa();
                    $ihfak = round(strlen($fdjk) / rand(2, 20));

                    //Adding hashvalue of domain and upload path ($dmndsasd) at the position $ihfak in string $fdjk
                    $fdjk = substr_replace($fdjk, $dmndsasd, $ihfak, 0);

                    //decoding the value and writing it in the file. If file has not write permission then updaing it in the table.
                    $decoded = base64_decode($fdjk);
                    if (is_writable($dir . 'license')) {
                        file_put_contents($dir . 'license', $decoded);
                    } else {
                        $fdjk = str_rot13($fdjk);
                        $opt_val = base64_decode('bGNkamthc2pka3NhY2w=');
                        update_option($opt_val, $fdjk);
                    }
                    update_option('lcwrtlfsaml', true);
                    wpWrapper::mo_sps__show_success_notice("Customer retrieved successfully");
                } else {
                    delete_option('sps_lk');
                    wpWrapper::mo_sps__show_error_notice("License key for this instance is incorrect. Make sure you have not tampered with it at all. Please enter a valid license key.");
                }
            } else
                wpWrapper::mo_sps__show_success_notice("Customer retrieved successfully");

            } else {
                wpWrapper::mo_sps__show_error_notice("Invalid username or password. Please try again.");
            }

            update_option('mo_saml_admin_password', '');
            
    }

    private function mo_sps_verify_license(){
        check_admin_referer('mo_sps_verify_license_option');

        if (empty($_POST['mo_sps_licence_key'])) {
            wpWrapper::mo_sps__show_error_notice('All the fields are required. Please enter valid license key.');
            return;
        }

        $code = htmlspecialchars(trim($_POST['mo_sps_licence_key']));
        $customer = new CustomerMOSPS();
        $content = $customer->mo_sps_check_customer_ln();
        if (!$content)
            return;
        
        $content = json_decode($content, true);
        if (strcasecmp($content['status'], 'SUCCESS') == 0) {


            $users_count = 0;
            if(array_key_exists('noOfUsers', $content)){
                $users_count = $content['noOfUsers'];
            }

            // if($users_count == 0){
            
            //     $key=wpWrapper::mo_sps_get_option('mo_sps_customer_token');

            //     wpWrapper::mo_sps_set_option('mo_sps_users_count',wpWrapper::mo_sps_encrypt_data($users_count,$key));

            //     wpWrapper::mo_sps__show_error_notice('You have purchased the license for '.$users_count. ' users. Contact us at samlsupport@xecurify.com to upgrade license for more users.');

            //     return;
            // }

            $content = $customer->mo_sps_vl($code, false);
            if (!$content)
                return;
            $content = json_decode($content, true);
            update_option('mo_sps_vl_check_t', time());
            if (is_array($content) and strcasecmp($content['status'], 'SUCCESS') == 0) {
                $key = get_option('mo_sps_customer_token');
                update_option('sps_lk', wpWrapper::mo_sps_encrypt_data($code, $key));
                update_option('site_ck_l', wpWrapper::mo_sps_encrypt_data("true", $key));
                wpWrapper::mo_sps_set_option('mo_sps_users_count',wpWrapper::mo_sps_encrypt_data($users_count,$key));
                update_option('t_site_status', wpWrapper::mo_sps_encrypt_data("false", $key));

                $dir = plugin_dir_path(__FILE__);

                //home url of the site
                $home_url = home_url();
                $home_url = trim($home_url, '/');
                if (!preg_match('#^http(s)?://#', $home_url)) {
                    $home_url = 'http://' . $home_url;
                }
                $urlParts = parse_url($home_url);
                $domain = preg_replace('/^www\./', '', $urlParts['host']);

                //upload path of the directory
                $upload_dir = wp_upload_dir();
                $asdjka = $domain . '-' . $upload_dir['basedir'];

                //Getting hash of the domain and upload path.
                $dmndsasd = hash_hmac('sha256', $asdjka, '4DHfjgfjasndfsajfHGJ');

                //finding the position where we need to insert this hash
                $fdjk = wpWrapper::djkasjdksa();
                $ihfak = round(strlen($fdjk) / rand(2, 20));

                //Adding hashvalue of domain and upload path ($dmndsasd) at the position $ihfak in string $fdjk
                $fdjk = substr_replace($fdjk, $dmndsasd, $ihfak, 0);

                //decoding the value and writing it in the file. If file has not write permission then updaing it in the table.
                $decoded = base64_decode($fdjk);
                if (is_writable($dir . 'license')) {
                    file_put_contents($dir . 'license', $decoded);
                } else {
                    $fdjk = str_rot13($fdjk);
                    $opt_val = base64_decode('bGNkamthc2pka3NhY2w=');
                    update_option($opt_val, $fdjk);
                }
                update_option('lcwrtlfsaml', true);

                $url = add_query_arg(array('tab' => 'general'), $_SERVER['REQUEST_URI']);
                wpWrapper::mo_sps__show_success_notice("Your license is verified. You can now setup the plugin.");
            } else if (is_array($content) and strcasecmp($content['status'], 'FAILED') == 0) {
                if (strcasecmp($content['message'], 'Code has Expired') == 0) {
                    wpWrapper::mo_sps__show_error_notice('License key you have entered has already been used. Please enter a key which has not been used before on any other instance or if you have exausted all your keys then contact us at samlsupport@xecurify.com to buy more');
                } else {
                    wpWrapper::mo_sps__show_error_notice('You have entered an invalid license key. Please enter a valid license key.');
                }
            } else {
                wpWrapper::mo_sps__show_error_notice('An error occurred while processing your request. Please Try again.');
            }
        } else {
            $key = get_option('mo_sps_customer_token');
            update_option('site_ck_l', wpWrapper::mo_sps_encrypt_data("false", $key));
            wpWrapper::mo_sps__show_error_notice('You have not upgraded yet.');
        }

    }

    private function mo_sps_go_back_from_license_page(){
        check_admin_referer('mo_sps_go_back_from_license_page_option');

        wpWrapper::mo_sps_deactivate();
        add_option('mo_sps_registration_status', 'removed_account');

    }

    private function mo_sps_remove_account(){
        check_admin_referer('mo_sps_remove_account_option');

        wpWrapper::mo_sps_deactivate();
        add_option('mo_sps_registration_status', 'removed_account');
    }

}