<?php

namespace MoSharePointObjectSync\Controller;

use MoSharePointObjectSync\API\Azure;
use MoSharePointObjectSync\Wrappers\pluginConstants;
use MoSharePointObjectSync\Wrappers\wpWrapper;

class appConfig{

    private static $instance;

    public static function getController(){
        if(!isset(self::$instance)){
            $class = __CLASS__;
            self::$instance = new $class;
        }
        return self::$instance;
    }

    public function mo_sps_save_settings(){
        $option = sanitize_text_field($_POST['option']);
        switch ($option){
            case 'mo_sps_client_config_option':{
                $this->mo_sps_save_client_config();
                break;
            }
            case 'mo_sps_client_sync_users_option':{
                $this->mo_sps_sync_users();
                break;
            }
            
            case 'mo_sps_upn_config_option':{
                $this->mo_sps_save_upn_config();
                break;
            }

            case 'mo_sps_profile_config_option':{
                $this->mo_sps_save_profile_config_option();
                break;
            }

            case 'mo_sps_client_wp_sync_individual_user_option':{
                $this->mo_sps_sync_individual_user();
                break;
            }
        }
    }

    private function mo_sps_check_for_empty_or_null(&$input,$arr){
        foreach ($arr as $key){
            if(!isset($_POST[$key]) || empty($_POST[$key])){
                return false;
            }
            $input[$key] = sanitize_text_field($_POST[$key]);
        }
        return $input;
    }

    private function mo_sps_save_client_config(){
        check_admin_referer('mo_sps_client_config_option');
        $input_arr = ['client_id','client_secret','admin_uri','tenant_id'];
        $sanitized_arr = [];
        if(!$this->mo_sps_check_for_empty_or_null($sanitized_arr,$input_arr)){
            wpWrapper::mo_sps__show_error_notice(esc_html__("Input is empty or present in the incorrect format."));
            return;
        }
        $sanitized_arr['upn_id'] = isset($_POST['upn_id'])?sanitize_text_field($_POST['upn_id']):'';
        $sanitized_arr['client_secret'] = wpWrapper::mo_sps_encrypt_data($sanitized_arr['client_secret'],hash("sha256",$sanitized_arr['client_id']));
        wpWrapper::mo_sps_set_option("mo_sps_application_config",$sanitized_arr);
        wpWrapper::mo_sps__show_success_notice(esc_html__("Settings Saved Successfully."));
    }






    private function mo_sps_save_upn_config(){
        check_admin_referer('mo_sps_upn_config_option');
        $config = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);
        $config['upn_id'] = isset($_POST['upn_id'])?sanitize_text_field($_POST['upn_id']):'';
        wpWrapper::mo_sps_set_option("mo_sps_application_config",$config);
        wpWrapper::mo_sps__show_success_notice(esc_html__("Settings Saved Successfully."));
    }

    private function mo_sps_save_profile_config_option(){
        check_admin_referer('mo_sps_profile_config_option');
        $input_arr = ['user_login','email','first_name','last_name','display_name'];
        $sanitized_arr = [];
        if(!$this->mo_sps_check_for_empty_or_null($sanitized_arr,$input_arr)){
            wpWrapper::mo_sps__show_error_notice(esc_html__("Input is empty or present in the incorrect format."));
            return;
        }
        wpWrapper::mo_sps_set_option("mo_sps_profile_mapping",$sanitized_arr);
        wpWrapper::mo_sps__show_success_notice(esc_html__("Settings Saved Successfully."));
    }




    private function mo_sps_sync_users(){
        check_admin_referer('mo_sps_client_sync_users_option');
        $data = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);
        if(!$data || empty($data)){
            return;
        }
        $azure_client = Azure::getClient($data);
        $azure_client->mo_sps_sync_azure_users();
        wpWrapper::mo_sps__show_success_notice(esc_html__("Users synced successfully."));
    }

    
    private function mo_sps_sync_individual_user(){
        check_admin_referer('mo_sps_client_wp_sync_individual_user_option');

        $config = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);
        
       

            $upn_id = $config['upn_id'];

        if(empty($upn_id)){
            wpWrapper::mo_sps__show_error_notice(esc_html__("Please provide UserPrincipalName/ID of user."));
            return;
        }


       
        if(!$config || empty($config)){
            return;
        }
        $share_client = Azure::getClient($config);
       
        $status = $share_client->mo_sps_sync_individual_azure_user(urlencode($upn_id));
       
        if(is_array($status)){
            if(isset($status['user_login_too_long']))
                wpWrapper::mo_sps__show_error_notice(esc_html__($status['user_login_too_long'][0]));
        }

        if(is_bool($status))
            wpWrapper::mo_sps__show_success_notice(esc_html__("User synced successfully."));

        
            
            
    }
}