<?php

namespace MoSharePointObjectSync\Observer;

use MoSharePointObjectSync\API\Azure;

use MoSharePointObjectSync\Wrappers\pluginConstants;
use MoSharePointObjectSync\Wrappers\sharepointWrapper;
use MoSharePointObjectSync\Wrappers\wpWrapper;
use MoSharePointObjectSync\API\CustomerMOSPS;

class adminObserver{

    private static $obj;

    public static function getObserver(){
        if(!isset(self::$obj)){
            self::$obj = new adminObserver();
        }
        return self::$obj;
    }

    public function mo_sps_admin_observer(){
        if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'testSPSApp'){
            $config = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);

            
            $client = Azure::getClient($config);
            $access_token = $client->mo_sps_access_token_details();
            $user_details = $client->mo_sps_get_specific_user_detail();
            $user_details = sharepointWrapper::mo_sps_array_get_sharepoint_user_profile($user_details);

            $this->mo_sps_display_test_attributes($user_details);
        }


        if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'testSPSUser'){

            $config = wpWrapper::mo_sps_get_option(pluginConstants::APP_CONFIG);
            
            
            if(!isset($config['upn_id']) || empty($config['upn_id'])){
            
                $error_code = [
                     "Error" => "EmptyUPN",
                    "Description" => "UPN is not configured in the plugin or incorrect."
                ];
                $this->mo_sps_display_error_message($error_code);
            }
            $client = Azure::getClient($config);
            $user_details = $client->mo_sps_get_specific_user_detail();
           
            if(isset($user_details['error'])){
                $error_code = [
                    "Error" => $user_details['error']['code'],
                    "Description" => is_array($user_details['error']['message'])?$user_details['error']['message']['value']:$user_details['error']['message']
                ];
                $this->mo_sps_display_error_message($error_code);
            }elseif(isset($user_details['error_description'])){
                $error_code = [
                    "Error" => '-',
                    "Description" => $user_details['error_description']
                ];
                $this->mo_sps_display_error_message($error_code);
            }

            $user_details = sharepointWrapper::mo_sps_array_get_sharepoint_user_profile($user_details);

            $this->mo_sps_display_fetch_attributes($user_details);
        }

        if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'mo_sps_contact_us_query_option'){
            $submited = $this->mo_sps_send_support_query();
            if(!is_null($submited)){
                if ( $submited == false ) {
                    wpWrapper::mo_sps__show_error_notice(esc_html__("Your query could not be submitted. Please try again."));
                } else {
                    wpWrapper::mo_sps__show_success_notice(esc_html__("Thanks for getting in touch! We shall get back to you shortly."));
                }
            }
        }
        if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'mo_sps_feedback'){
            $submited = $this->mo_sps_send_email_alert();
            if( json_last_error() == JSON_ERROR_NONE) {
                if(is_array( $submited ) && array_key_exists( 'status', $submited ) && $submited['status'] == 'ERROR' ) {
                    wpWrapper::mo_sps__show_error_notice(esc_html__($submited['message']));
                }
                else{
                    if( $submited == false ){
                        wpWrapper::mo_sps__show_error_notice(esc_html__("Error while submitting the query."));
                    }
                }
            }

            include_once(ABSPATH . 'wp-admin/includes/plugin.php');
           
            deactivate_plugins( MO_SPS_PLUGIN_FILE );
            wpWrapper::mo_sps__show_success_notice(esc_html__("Thank you for the feedback."));
            
        }
        if(isset($_REQUEST['option']) && $_REQUEST['option'] == 'mo_sps_skip_feedback'){


            $submited = $this->mo_sps_send_email_alert(true);

           
            if ( json_last_error() == JSON_ERROR_NONE ) {
                if ( is_array( $submited ) && array_key_exists( 'status', $submited ) && $submited['status'] == 'ERROR' ) {
                    wpWrapper::mo_sps__show_error_notice(esc_html__($submited['message']));
                }
                else {
                    if ( $submited == false ) {
                        wpWrapper::mo_sps__show_error_notice(esc_html__("Error while submitting the query."));
                    }
                }
            }

            wpWrapper::mo_sps__show_success_notice(esc_html__("Plugin deactivated successfully."));
            include_once(ABSPATH . 'wp-admin/includes/plugin.php');
           
            deactivate_plugins( MO_SPS_PLUGIN_FILE );
        }
    }
    private function mo_sps_send_email_alert($isSkipped = false){
        $user = wp_get_current_user();

        $message = 'Plugin Deactivated';

        $deactivate_reason_message = array_key_exists( 'query_feedback', $_POST ) ? htmlspecialchars($_POST['query_feedback']) : false;

        if($isSkipped)
            $deactivate_reason_message = "skipped";

        $reply_required = '';
        if(isset($_POST['get_reply']))
            $reply_required = htmlspecialchars($_POST['get_reply']);
        if(empty($reply_required)){
            $reply_required = "don't reply";
            $message.='<b style="color:red";> &nbsp; [Reply :'.$reply_required.']</b>';
        }else{
            $reply_required = "yes";
            $message.='[Reply :'.$reply_required.']';
        }

        if(is_multisite())
            $multisite_enabled = 'True';
        else
            $multisite_enabled = 'False';

        $message.= ', [Multisite enabled: ' . $multisite_enabled .']';
        
        $message.= ', Feedback : '.$deactivate_reason_message.'';
            
        $email = '';
        $rate_value = '';

        if (isset($_POST['rate']))
            $rate_value = htmlspecialchars($_POST['rate']);
        
        $message.= ', [Rating :'.$rate_value.']';

        if (isset($_POST['query_mail']))
            $email = $_POST['query_mail'];

        if(!filter_var($email, FILTER_VALIDATE_EMAIL)){
            $email = get_option('mo_saml_admin_email');
            if(empty($email))
                $email = $user->user_email;
        }
        $phone = get_option( 'mo_saml_admin_phone' );
        $feedback_reasons = new CustomerMOSPS();

        $response = json_decode( $feedback_reasons->mo_sps_send_email_alert( $email, $phone, $message ), true );

        return $response;

    }
    private function mo_sps_send_support_query(){
        $email    = sanitize_email($_POST['mo_sps_contact_us_email']);
        $phone    = htmlspecialchars($_POST['mo_sps_contact_us_phone']);
        $query    = htmlspecialchars($_POST['mo_sps_contact_us_query']);

        $query = '[Embed sharepoint onedrive documents plugin] ' . $query;
                
        $customer = new CustomerMOSPS();
  
        $response = $customer->mo_sps_submit_contact_us($email,$phone,$query);

        return $response;
    }
    public function mo_sps_display_error_message($error_code){
        ?>
            

            <div style="width:100%;display:flex;flex-direction:column;justify-content:center;align-items:center;font-size:15px;margin-top:10px;width:100%;">
                
                <div style="width:86%;padding: 15px;text-align: center;background-color:#f2dede;color:#a94442;border: 1px solid #E6B3B2;font-size: 18pt;margin-bottom:20px;">
                    Error
                </div>

                <table class="mo-ms-tab-content-app-config-table" style="border-collapse:collapse;width:90%">
                    <tr>
                        <td style="padding: 30px 5px 30px 5px;border:1px solid #757575;" colspan="2"><h2><span>Test Configuration Failed</span></h2></td>
                    </tr>
                    <?php foreach ($error_code as $key => $value){
                       echo '<tr><td style="padding: 30px 5px 30px 5px;border:1px solid #757575;" class="left-div"><span style="margin-right:10px;"><b>'.esc_html($key).':</b></span></td>
                       <td style="padding: 30px 5px 30px 5px;border:1px solid #757575;" class="right-div"><span>'.esc_html($value).'</span></td></tr>';
                    }?>
                </table>
                <h3 style="margin:20px;">
                    Contact us at <a style="color:#dc143c" href="mailto:samlsupport@xecurify.com">samlsupport@xecurify.com</a>
                </h3>
            </div>
        <?php
        exit();
    }
    private function mo_sps_display_test_attributes($details){
        ?>
        <div style="width:100%;height:100%;display:flex;align-items:center;flex-direction:column;border:1px solid #eee;padding:10px;">
       
                <div style="width:90%;color: #3c763d;background-color: #dff0d8;padding: 2%;margin-bottom: 20px;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
                    Success
                </div>
			<div style="display:block;text-align:center;margin-bottom:4%;">
                <svg class="animate" width="100" height="100">
                    <filter id="dropshadow" height="">
                    <feGaussianBlur in="SourceAlpha" stdDeviation="3" result="blur"></feGaussianBlur>
                    <feFlood flood-color="rgba(76, 175, 80, 1)" flood-opacity="0.5" result="color"></feFlood>
                    <feComposite in="color" in2="blur" operator="in" result="blur"></feComposite>
                    <feMerge> 
                        <feMergeNode></feMergeNode>
                        <feMergeNode in="SourceGraphic"></feMergeNode>
                    </feMerge>
                    </filter>
				
				    <circle cx="50" cy="50" r="46.5" fill="none" stroke="rgba(76, 175, 80, 0.5)" stroke-width="5"></circle>
				    <path d="M67,93 A46.5,46.5 0,1,0 7,32 L43,67 L88,19" fill="none" stroke="rgba(76, 175, 80, 1)" stroke-width="5" stroke-linecap="round" stroke-dasharray="80 1000" stroke-dashoffset="-220" style="filter:url(#dropshadow)"></path>
			    </svg>
              
              <div></div>
              
                <div style="margin:3%;margin-top:12%;display:flex;text-align:center;">

                <div style="margin-right:20px;">
                <input class="mo-ms-tab-content-button" style="box-shadow:none!imporat;height:30px;background-color: #1B9BA1;border-color: #1B9BA1;color: #FFF;cursor: pointer;" type="button" value="Sync Documents" onClick="close_and_redirect_to_document_sync();"> &nbsp;
                </div>
                <div>
		        <input class="mo-ms-tab-content-button" style="box-shadow:none!imporat;height:30px;background-color: #1B9BA1;border-color: #1B9BA1;color: #FFF;cursor: pointer;" type="button" value="SharePoint User Profile" onClick="close_and_redirect_to_sync_user();">
                </div>
            

                </div>
                
              <style>
			  svg.animate path {
			  animation: dash 1.5s linear both;
			  animation-delay: 1s;
			}
			  @keyframes dash {
			  0% { stroke-dashoffset: 210; }
			  75% { stroke-dashoffset: -220; }
			  100% { stroke-dashoffset: -205; }
			}
			</style>
        </div>


                
            </div>
            
        </div>
        
        <script>
             function close_and_redirect_to_sync_user(){
                
                 window.opener.redirect_to_sync_users();
                 self.close();
             }  
             function close_and_redirect_to_document_sync(){
                window.opener.redirect_to_document_sync();
                self.close();
            }  
        </script>

       
        <?php
        $this->load_css();
        exit();
    }

    private function mo_sps_display_fetch_attributes($details){
        ?>

        <div style="display:flex;justify-content:center;align-items:center;flex-direction:column;border:1px solid #eee;padding:10px;">


                <div style="width:90%;color: #3c763d;background-color: #dff0d8;padding: 2%;margin-bottom: 20px;text-align: center;border: 1px solid #AEDB9A;font-size: 18pt;">
                    Success
                </div>
				<div style="display:block;text-align:center;margin-bottom:4%;"><svg class="animate" width="100" height="100">
				<filter id="dropshadow" height="">
				  <feGaussianBlur in="SourceAlpha" stdDeviation="3" result="blur"></feGaussianBlur>
				  <feFlood flood-color="rgba(76, 175, 80, 1)" flood-opacity="0.5" result="color"></feFlood>
				  <feComposite in="color" in2="blur" operator="in" result="blur"></feComposite>
				  <feMerge> 
					<feMergeNode></feMergeNode>
					<feMergeNode in="SourceGraphic"></feMergeNode>
				  </feMerge>
				</filter>
				
				<circle cx="50" cy="50" r="46.5" fill="none" stroke="rgba(76, 175, 80, 0.5)" stroke-width="5"></circle>
				
				<path d="M67,93 A46.5,46.5 0,1,0 7,32 L43,67 L88,19" fill="none" stroke="rgba(76, 175, 80, 1)" stroke-width="5" stroke-linecap="round" stroke-dasharray="80 1000" stroke-dashoffset="-220" style="filter:url(#dropshadow)"></path>
			  </svg><style>
			  svg.animate path {
			  animation: dash 1.5s linear both;
			  animation-delay: 1s;
			}
			  @keyframes dash {
			  0% { stroke-dashoffset: 210; }
			  75% { stroke-dashoffset: -220; }
			  100% { stroke-dashoffset: -205; }
			}
			</style></div>
                

                <div style="border-top:1px solid #eee;width:95%;"></div>

                <div class="test-container" style="margin-top:10px;background:#fff;">
                    <table class="mo-ms-tab-content-app-config-table">
                        <tr>
                            <td style="align:center;" colspan="2">
                                <span><h2>Test Attributes:</h2></span>
                            </td>
                        </tr>
                <?php
                    foreach ($details as $key => $value){
                    if(!is_array($value) && !empty($value)){
                    ?>
                    <tr>
                        <td class="left-div"><span><?php echo esc_html($key);?></span></td>
                        <td class="right-div"><span><?php echo esc_html($value);?></span></td>
                    </tr>
                    <?php
                    }
                 }
                ?>
                    </table>
                </div>
        </div>
                </div>
        
        <?php
        $this->load_css();
        exit();
    }

    private function load_css(){
        ?>
        <style>
            .test-container{
                width: 100%;
                background: #f1f1f1;
                margin-top: -30px;
            }

            .mo-ms-tab-content-app-config-table{
                max-width: 1000px;
                background: white;
                padding: 1em 2em;
                margin: 2em auto;
                border-collapse:collapse;
                border-spacing:0;
                display:table;
                font-size:14pt;
                
            }
         

            .mo-ms-tab-content-app-config-table td.left-div{
                width: 40%;
                word-break: break-all;
                font-weight:bold;
                border:2px solid #949090;
                padding:2%;
            }
            .mo-ms-tab-content-app-config-table td.right-div{
                width: 40%;
                word-break: break-all;
                padding:2%;
                border:2px solid #949090;
                word-wrap:break-word;
            }

        </style>
        <?php
    }
}