<?php

/*
Plugin Name: Embed SharePoint OneDrive Documents
Plugin URI: https://plugins.miniorange.com/
Description: This plugin will allow you to sync users/files/sites from SharePoint Online/Office 365 to wordpress.
Version: 1.0.3
Author: miniOrange
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/

namespace MoSharePointObjectSync;
require_once __DIR__ . '/vendor/autoload.php';

use MoSharePointObjectSync\View\adminView;
use MoSharePointObjectSync\Controller\adminController;
use MoSharePointObjectSync\Observer\adminObserver;
use MoSharePointObjectSync\Observer\documentObserver;
use MoSharePointObjectSync\View\feedbackForm;

define('MO_SPS_PLUGIN_FILE',__FILE__);
define('MO_SPS_PLUGIN_DIR',__DIR__.DIRECTORY_SEPARATOR);

class MOsps{

    private static $instance;
    public static $version = "1.0.3";

    public static function mo_sps_load_instance(){
        if(!isset(self::$instance)){
            $class = __CLASS__;
            self::$instance = new $class;
            self::$instance->mo_sps_load_hooks();
        }
        return self::$instance;
    }

    public function mo_sps_load_hooks(){
        add_action('admin_menu',[$this,'mo_sps_admin_menu']);
        add_action( 'admin_enqueue_scripts', [$this, 'mo_sps_settings_style' ] );
        add_action('admin_init',[adminController::getController(),'mo_sps_admin_controller']);
        add_action( 'admin_footer', [feedbackForm::getView() , 'mo_sps_display_feedback_form'] );
        add_action('init',[adminObserver::getObserver(),'mo_sps_admin_observer']);
        // add_action('init',[documentObserver::getObserver(),'mo_sps_document_observer']);

        add_action( "wp_ajax_mysyncaction", [documentObserver::getObserver(),'mo_sps_document_observer'] );
        add_action( "wp_ajax_nopriv_mysyncaction", [documentObserver::getObserver(),'mo_sps_document_observer'] );


        add_action( 'admin_enqueue_scripts', array( $this, 'mo_sps_settings_script' ) );
        register_uninstall_hook(__FILE__, 'mo_sps_uninstall');
    }

	


    public function mo_sps_admin_menu(){
        $page = add_menu_page(
            'miniOrange SharePoint Integration ' .__('+ Sync'),
            'miniOrange SharePoint Integration',
            'administrator',
            'mo_sps',
            [adminView::getView(),'mo_sps_menu_display'],
            plugin_dir_url( __FILE__ ) . 'images/miniorange.png'
        );
    }

    function mo_sps_settings_style($page){
        if( $page != 'toplevel_page_mo_sps'){
            return;
        }
        $css_url = plugins_url('includes/css/mo_sps_settings.css',__FILE__);
        $css_phone_url = plugins_url('includes/css/phone.css',__FILE__);
        $css_jquery_ui_url = plugins_url('includes/css/jquery-ui.css',__FILE__);

        wp_enqueue_style('mo_sps_css',$css_url,array(),self::$version);
        wp_enqueue_style('mo_sps_phone_css',$css_phone_url,array(),self::$version);
        wp_enqueue_style('mo_sps_jquery_ui_css',$css_jquery_ui_url,array(),self::$version);
    }

    function mo_sps_settings_script($page){
        $phone_js_url = plugins_url('includes/js/phone.js',__FILE__);
        $setting_js_url= plugins_url('includes/js/settings.js',__FILE__);
        wp_enqueue_script('mo_sps_phone_js',$phone_js_url,array(),self::$version);
        wp_enqueue_script('mo_settings_js',$setting_js_url,array(),self::$version);

    }

    function so_enqueue_scripts(){
        wp_register_script( 
          'ajaxHandle', 
          plugins_url('includes/js/ajax.js', __FILE__), 
          array(), 
          false, 
          true 
        );
        wp_enqueue_script( 'ajaxHandle' );
        wp_localize_script( 
          'ajaxHandle', 
          'ajax_object', 
          array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) 
        );
      }
    

}
MOsps::mo_sps_load_instance();